/*
 * Copyright (c) 2003 shimitei
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <windows.h>
#include "qt_os.h"
#include "qt_type.h"
#include "spi00in.h"

#pragma comment(lib , "qtmlclient.lib")


bool g_QTavailable;
bool g_QTchecked;
//---------------------------------------------------------------------------
bool QT_init(void)
{
	if (InitializeQTML(0L) != 0)
	{
		return false;
	}

	if (EnterMovies() != 0)
	{
		return false;
	}

	return true;
}
//---------------------------------------------------------------------------
bool __forceinline QT_check(void)
{
	if (g_QTchecked) return g_QTavailable;

	g_QTavailable = QT_init();
	g_QTchecked = true;
	return g_QTavailable;
}
//---------------------------------------------------------------------------
void QT_quit()
{
	ExitMovies();
	TerminateQTML();
}
//---------------------------------------------------------------------------
int IsFileType(const char *data, const char *filename);
int GetMovieFrame(char *filename, HANDLE *pHBInfo, HANDLE *pHBm);
int GetPictureEx(const LPSTR buf, const long len, const unsigned int flag,
                   HANDLE *pHBInfo, HANDLE *pHBm,
                   const SPI_PROGRESS lpPrgressCallback,
                   const long lData, const bool IsPreview);
//---------------------------------------------------------------------------
int WINAPI DllMain(HINSTANCE hinst, unsigned long reason, void*)
{
    switch (reason) {
        case DLL_PROCESS_ATTACH:
			g_QTchecked = false;
            break;
        case DLL_PROCESS_DETACH:
            QT_quit();
            break;
    }

    return 1;
}
//---------------------------------------------------------------------------
//t@C̎ނ𔻒
//data     : t@C擪̃f[^
//filename : t@C
//return   : t@C^CvԂ
int IsFileType(const char *data, const char *filename)
{
    int filetype = FT_UNKNOWN;
    UNKNOWN_FILE *mm_h = (UNKNOWN_FILE *)data;
#if 0
	if (filename) {
		int len = strlen(filename);
		if (len > 4) {
			DWORD ext = *(DWORD*)(filename +len -4);
			ext |= 0x20202000;
			if (ext == 0x766f6d2e/*".mov"*/) return FT_MOV;
		}
	}
#endif
    if (
        /* MOV */
          //playable[MOVPLAY] &&
        (   mm_h->uk2 == MDAT_TYPE_ID
         || mm_h->uk2 == PNOT_TYPE_ID
         || mm_h->uk2 == MOOV_TYPE_ID
         || mm_h->uk4 == MDAT_TYPE_ID
         || mm_h->uk4 == PNOT_TYPE_ID
         || mm_h->uk4 == MOOV_TYPE_ID )
              ) {
            filetype = FT_MOV;
#if 0
    } else if (
        /* AVI */
          //playable[AVIPLAY] &&
        (   mm_h->uk1 == RIFF_TYPE_ID
         && mm_h->uk3 == AVI_TYPE_ID )
              ) {
            filetype = FT_AVI;
#endif
    }

    return filetype;
}
//---------------------------------------------------------------------------
//̃t[DIBɕϊ
//filename : t@C
//pHBInfo  : SusieɕԂBITMAPINFO߂nh̃|C^
//pHBm     : SusieɕԂBITMAPf[^߂nh̃|C^
//return   : SusieG[R[h
int GetMovieFrame(char *filename, HANDLE *pHBInfo, HANDLE *pHBm)
{
	QToffscreen qt;

	if ( !qt.OpenMovie(filename) )
	{
		return SPI_OTHER_ERROR;
	}

	HGLOBAL hMem;
	if ( !qt.GetMoviePreview(&hMem) )
	{
		return SPI_OTHER_ERROR;
	}

	char *p = (char*)GlobalLock(hMem);
	BITMAPINFOHEADER *pbmi = (BITMAPINFOHEADER *)p;
	DWORD imgsize = ((pbmi->biWidth * (pbmi->biBitCount/8) +3) & ~3) * abs(pbmi->biHeight);
	DWORD infosize = 0x28;
	switch (pbmi->biBitCount) {
		case 1:
			infosize += sizeof(RGBQUAD) * 2;
			break;
		case 4:
			if (pbmi->biClrUsed == 0) infosize += sizeof(RGBQUAD) * 16;
			else infosize += sizeof(RGBQUAD) * pbmi->biClrUsed;
			break;
		case 8:
			if (pbmi->biClrUsed == 0) infosize += sizeof(RGBQUAD) * 256;
			else infosize += sizeof(RGBQUAD) * pbmi->biClrUsed;
			break;
		case 16:
			if (pbmi->biCompression == BI_BITFIELDS) infosize += sizeof(DWORD) * 3;
			break;
		case 24:
			break;
		case 32:
			if (pbmi->biCompression == BI_BITFIELDS) infosize += sizeof(DWORD) * 3;
			break;
		default:
			;
	}

	int ret;
    *pHBInfo = LocalAlloc(LMEM_MOVEABLE, infosize);
    *pHBm = LocalAlloc(LMEM_MOVEABLE, imgsize);
	if (*pHBInfo == NULL || *pHBm == NULL) {
		if (*pHBInfo) LocalFree(*pHBInfo);
		if (*pHBm) LocalFree(*pHBm);
		ret = SPI_NO_MEMORY;
	} else {
        char *pInfo = (char *)LocalLock(*pHBInfo);
        memcpy(pInfo, p, infosize);

        char *pBm = (char *)LocalLock(*pHBm);
        memcpy(pBm, p+infosize, imgsize);

        LocalUnlock(*pHBm);
        LocalUnlock(*pHBInfo);
		ret = SPI_ALL_RIGHT;
	}
	GlobalFree(hMem);

    return ret;
}
//---------------------------------------------------------------------------
int __stdcall GetPluginInfo(int infono, LPSTR buf, int buflen)
{
    char *infomation[] = {
    /* 0     */  "00IN",
    /* 1     */  "MOV Preview v0.0.1",
    /* 2*0+2 */  "*.mov",
    /* 2*0+3 */  "QuickTime Movie file",
    };

    if (infono < 0) return 0;
    if (infono < (sizeof(infomation) / sizeof(infomation[0]))) {
        //񎁐݊
        //K'\0'H
        lstrcpyn(buf, infomation[infono], buflen);
        //'\0'͐Ȃ
        return lstrlen(buf);
    }

    return 0;
}
//---------------------------------------------------------------------------
int __stdcall IsSupported(LPSTR filename, DWORD dw)
{
if (!QT_check()) return 0;

    char h_buf[HEAD_BUFSIZE];
    char *data;
    int filetype;

    if ((dw & 0xFFFF0000) == 0) {
    /* t@Cnh */
        DWORD ReadBytes;
        if (!ReadFile((HANDLE)dw, h_buf, HEAD_BUFSIZE, &ReadBytes, NULL)) {
            return 0;
        }
        data = h_buf;
    } else {
    /*  */
        data = (char *)dw;
    }

    /* Ήt@CmF */
    filetype = IsFileType(data, filename);
    if (filetype != FT_UNKNOWN) {
        return 1;
    }

    return 0;
}
//---------------------------------------------------------------------------
int __stdcall GetPictureInfo
(LPSTR buf, long len, unsigned int flag, struct PictureInfo *lpInfo)
{
if (!QT_check()) return SPI_NO_FUNCTION;

    char h_buf[HEAD_BUFSIZE];
    char *filename;
    char *data;

    if ((flag & 7) == 0) {
    /* t@C */
        HANDLE hf;
        DWORD ReadBytes;
        hf = CreateFile(buf, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
        if (hf == INVALID_HANDLE_VALUE)
            return SPI_FILE_READ_ERROR;
        //SetFilePointer(hf, len, NULL, FILE_BEGIN);//MacoCiɂ͑ΉȂ
        if (!ReadFile(hf, h_buf, HEAD_BUFSIZE, &ReadBytes, NULL)) {
            CloseHandle(hf);
            return SPI_FILE_READ_ERROR;
        }
        CloseHandle(hf);
        filename = buf;
        data = h_buf;
    } else {
    /*  */
        data = buf;
        filename = NULL;
    }

    /* Ήt@CmF */
    int filetype = IsFileType(data, filename);
    if (filetype == FT_UNKNOWN) return SPI_NOT_SUPPORT;

    //ACR(Œ)Ԃ
    lpInfo->left       = 0;
    lpInfo->top        = 0;
    lpInfo->width      = 32;
    lpInfo->height     = 32;
    lpInfo->x_density  = 0;
    lpInfo->y_density  = 0;
    lpInfo->colorDepth = 24;
    lpInfo->hInfo      = NULL;

    return SPI_ALL_RIGHT;
}
//---------------------------------------------------------------------------
//GetPreviewGetPicturȅ܂Ƃ߂
int GetPictureEx(
        const LPSTR buf, const long len, const unsigned int flag,
        HANDLE *pHBInfo, HANDLE *pHBm,
        const SPI_PROGRESS lpPrgressCallback,
        const long lData, const bool IsPreview)
{
    char h_buf[HEAD_BUFSIZE];
    char *data;
    char *filename;

    if ((flag & 7) == 0) {
    /* t@C */
        HANDLE hf;
        DWORD ReadBytes;
        hf = CreateFile(buf, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
        if (hf == INVALID_HANDLE_VALUE)
            return SPI_FILE_READ_ERROR;
        //SetFilePointer(hf, len, NULL, FILE_BEGIN);//MacoCiΉ
        if (!ReadFile(hf, h_buf, HEAD_BUFSIZE, &ReadBytes, NULL)) {
            CloseHandle(hf);
            return SPI_FILE_READ_ERROR;
        }
        CloseHandle(hf);
        filename = buf;
        data = h_buf;
    } else {
    /*  */
        return SPI_NOT_SUPPORT;
    }

    /* Ήt@CmF */
    int filetype;
    filetype = IsFileType(data, filename);
    if (filetype == FT_UNKNOWN) return SPI_NOT_SUPPORT;

#if 0
    //'킴ƃG['Ԃ
    if (!IsPreview) {
        return SPI_OTHER_ERROR;
    }
#endif

    return GetMovieFrame(filename, pHBInfo, pHBm);
}
//---------------------------------------------------------------------------
int __stdcall SusieGetPicture(
        LPSTR buf, long len, unsigned int flag,
        HANDLE *pHBInfo, HANDLE *pHBm,
        SPI_PROGRESS lpPrgressCallback, long lData)
{
if (!QT_check()) return SPI_NO_FUNCTION;

    if (lpPrgressCallback != NULL)
        if (lpPrgressCallback(0, 1, lData)) // 0%
            return SPI_ABORT;

	int ret = GetPictureEx(buf, len, flag, pHBInfo, pHBm, lpPrgressCallback, lData, false);

    if (lpPrgressCallback != NULL)
        if (lpPrgressCallback(1, 1, lData)) // 100%
            return SPI_ABORT;

	return ret;
}
//---------------------------------------------------------------------------
int __stdcall GetPreview(
    LPSTR buf, long len, unsigned int flag,
    HANDLE *pHBInfo, HANDLE *pHBm,
    SPI_PROGRESS lpPrgressCallback, long lData)
{
if (!QT_check()) return SPI_NO_FUNCTION;

    if (lpPrgressCallback != NULL)
        if (lpPrgressCallback(0, 1, lData)) // 0%
            return SPI_ABORT;

    int ret = GetPictureEx(buf, len, flag, pHBInfo, pHBm, lpPrgressCallback, lData, true);

    if (lpPrgressCallback != NULL)
        if (lpPrgressCallback(1, 1, lData)) // 100%
            return SPI_ABORT;

    return ret;
}
//---------------------------------------------------------------------------
